@@ -29,7 +29,6 @@
 #include <grp.h>
 
 #define FUSE_DEV_NEW "/dev/fuse"
-#define FUSE_CONF "/etc/fuse.conf"
 
 #ifndef MS_DIRSYNC
 #define MS_DIRSYNC 128
@@ -37,7 +36,6 @@
 
 static const char *progname = "ntfs-3g-mount";
 
-static int user_allow_other = 0;
 static int mount_max = 1000;
 
 int drop_privs(void);
@@ -56,13 +54,6 @@ static const char *get_user_name(void)
 
 int drop_privs(void)
 {
-	if (!geteuid()) {
-		if (setgroups(0, NULL) < 0) {
-			perror("priv drop: setgroups failed");
-			return -1;
-		}
-	}
-	
 	if (!getegid()) {
 
 		gid_t new_gid = getgid();
@@ -180,59 +171,6 @@ static int add_mount(const char *source, const char *mnt, const char *type,
 }
 #endif /* IGNORE_MTAB */
 
-static void strip_line(char *line)
-{
-    char *s = strchr(line, '#');
-    if (s != NULL)
-        s[0] = '\0';
-    for (s = line + strlen(line) - 1; s >= line && isspace((unsigned char) *s); s--);
-    s[1] = '\0';
-    for (s = line; isspace((unsigned char) *s); s++);
-    if (s != line)
-        memmove(line, s, strlen(s)+1);
-}
-
-static void parse_line(char *line, int linenum)
-{
-    int tmp;
-    if (strcmp(line, "user_allow_other") == 0)
-        user_allow_other = 1;
-    else if (sscanf(line, "mount_max = %i", &tmp) == 1)
-        mount_max = tmp;
-    else if(line[0])
-        fprintf(stderr, "%s: unknown parameter in %s at line %i: '%s'\n",
-                progname, FUSE_CONF, linenum, line);
-}
-
-static void read_conf(void)
-{
-    FILE *fp = fopen(FUSE_CONF, "r");
-    if (fp != NULL) {
-        int linenum = 1;
-        char line[256];
-        int isnewline = 1;
-        while (fgets(line, sizeof(line), fp) != NULL) {
-            if (isnewline) {
-                if (line[strlen(line)-1] == '\n') {
-                    strip_line(line);
-                    parse_line(line, linenum);
-                } else {
-                    fprintf(stderr, "%s: reading %s: line %i too long\n",
-                            progname, FUSE_CONF, linenum);
-                    isnewline = 0;
-                }
-            } else if(line[strlen(line)-1] == '\n')
-                isnewline = 1;
-            if (isnewline)
-                linenum ++;
-        }
-        fclose(fp);
-    } else if (errno != ENOENT) {
-        fprintf(stderr, "%s: failed to open %s: %s\n", progname, FUSE_CONF,
-                strerror(errno));
-    }
-}
-
 static int begins_with(const char *s, const char *beg)
 {
     if (strncmp(s, beg, strlen(beg)) == 0)
@@ -276,7 +214,7 @@ static int find_mount_flag(const char *s, unsigned len, int *on, int *flag)
             *flag = mount_flags[i].flag;
             if (!mount_flags[i].safe && getuid() != 0) {
                 *flag = 0;
-                fprintf(stderr, "%s: unsafe option %s ignored\n",
+                fprintf(stderr, "%s: unsafe option '%s' ignored\n",
                         progname, opt);
             }
             return 1;
@@ -412,13 +350,6 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
                     skip_option = 1;
                 }
             }
-            if (getuid() != 0 && !user_allow_other &&
-                (opt_eq(s, len, "allow_other") ||
-                 opt_eq(s, len, "allow_root"))) {
-                fprintf(stderr, "%s: option %.*s only allowed if 'user_allow_other'"
-			" is set in /etc/fuse.conf\n", progname, len, s);
-                goto err;
-            }
             if (!skip_option) {
                 if (find_mount_flag(s, len, &on, &flag)) {
                     if (on)
@@ -489,17 +420,16 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
         *typep = type;
         *mnt_optsp = mnt_opts;
     }
-    free(optbuf);
-
-    return res;
-
- err:
+out:    
     free(fsname);
+    free(optbuf);
+    return res;
+err:
     free(source);
     free(type);
     free(mnt_opts);
-    free(optbuf);
-    return -1;
+    res = -1;
+    goto out;
 }
 
 static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
@@ -588,7 +518,13 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *currdir_fd,
 
 static int try_open(const char *dev, char **devp)
 {
-    int fd = open(dev, O_RDWR);
+    int fd;
+     
+    if (restore_privs())
+	return -1;
+    fd = open(dev, O_RDWR);
+    if (drop_privs())
+	return -1;
     if (fd != -1) {
         *devp = strdup(dev);
         if (*devp == NULL) {
@@ -638,12 +574,10 @@ static int mount_fuse(const char *mnt, const char *opts)
     if (fd == -1)
         return -1;
 
-    read_conf();
-
     if (getuid() != 0 && mount_max != -1) {
         if (count_fuse_fs() >= mount_max) {
-            fprintf(stderr, "%s: too many FUSE filesystems mounted; "
-		    "mount_max=N can be set in /etc/fuse.conf\n", progname);
+            fprintf(stderr, "%s: too many mounted FUSE filesystems (%d+)\n",
+		    progname, mount_max);
 	    goto err;
         }
     }
@@ -665,8 +599,14 @@ static int mount_fuse(const char *mnt, const char *opts)
 
     if (restore_privs())
 	goto err;
-    
+
     if (geteuid() == 0) {
+	
+	if (setgroups(0, NULL) == -1) {
+	    perror("priv drop: setgroups failed");
+	    goto err;
+	}
+	    
         res = add_mount(source, mnt, type, mnt_opts);
         if (res == -1) {
             umount2(mnt, 2); /* lazy umount */
@@ -724,5 +664,6 @@ int fusermount(int unmount, int quiet, int lazy, const char *opts,
 	    res = mount_fuse(mnt, opts);
 out:    
     umask(old_umask);
+    free(mnt);
     return res;	    
 }
